@@ -20,13 +20,13 @@
 
 QT_BEGIN_NAMESPACE
 
-class QDarwinMediaDevices;
+class QDarwinAudioDevices;
 
 class QDarwinFormatInfo : public QPlatformMediaFormatInfo
 {
 public:
     QDarwinFormatInfo();
-    ~QDarwinFormatInfo();
+    ~QDarwinFormatInfo() override;
 
     static int audioFormatForCodec(QMediaFormat::AudioCodec codec);
     static NSString *videoFormatForCodec(QMediaFormat::VideoCodec codec);
