@@ -16,7 +16,7 @@
 //
 
 #include <qmediametadata.h>
-#include <qffmpeg_p.h>
+#include <QtFFmpegMediaPluginImpl/private/qffmpeg_p.h>
 
 QT_BEGIN_NAMESPACE
 
