@@ -1,14 +1,14 @@
 // Copyright (C) 2024 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
-#include <QtTest/QtTest>
+#include <QtTest/qtest.h>
 #include <qmediaplayer.h>
 #include <qvideoframe.h>
 #include <qdebug.h>
 
-#include "mediafileselector.h"
-#include "mediabackendutils.h"
-#include "testvideosink.h"
+#include <private/mediafileselector_p.h>
+#include <private/mediabackendutils_p.h>
+#include <private/testvideosink_p.h>
 #include "private/qvideotexturehelper_p.h"
 #include "private/qvideowindow_p.h"
 #include <thread>
@@ -47,8 +47,8 @@ private:
     void addMediaPlayerFrameTestData(F &&f);
 
 private:
-    MaybeUrl m_oneRedFrameVideo = QUnexpect{};
-    MaybeUrl m_colorsVideo = QUnexpect{};
+    MaybeUrl m_oneRedFrameVideo{ QUnexpect{} };
+    MaybeUrl m_colorsVideo{ QUnexpect{} };
     MediaFileSelector m_mediaSelector;
 };
 
@@ -220,7 +220,7 @@ void tst_QVideoFrameBackend::toImage_rendersUpdatedFrame_afterMappingInWriteMode
 
 void tst_QVideoFrameBackend::toImage_returnsImage_whenCalledFromSeparateThreadAndWhileRenderingToWindow()
 {
-    if (qEnvironmentVariable("QTEST_ENVIRONMENT").toLower() == "ci") {
+    if (isCI()) {
 #ifdef Q_OS_MACOS
         QSKIP("SKIP on macOS because of crash and error \"Failed to create QWindow::MetalSurface. Metal is not supported by any of the GPUs in this system.\"");
 #elif defined(Q_OS_ANDROID)
@@ -257,7 +257,7 @@ void tst_QVideoFrameBackend::toImage_returnsImage_whenCalledFromSeparateThreadAn
     player.play();
 
     // assert
-    QTRY_COMPARE_GE_WITH_TIMEOUT(images.size(), 10u, 60 * 1000 /*ms*/ );
+    QTRY_COMPARE_GE_WITH_TIMEOUT(images.size(), 10u, std::chrono::seconds(60) );
 }
 
 QTEST_MAIN(tst_QVideoFrameBackend)
